@@ -17,13 +17,13 @@ class TestShowSwitch(unittest.TestCase):
         switch = self._makeOne()
         switch.off()
         self.assertEqual(switch.stack, [False])
-        
+
     def test_reset(self):
         switch = self._makeOne()
         switch.stack.append(False)
         switch.reset()
         self.assertEqual(switch.stack, [])
-            
+
     def test_call_true(self):
         switch = self._makeOne()
         self.assertEqual(switch(), True)
@@ -36,7 +36,7 @@ class TestShowSwitch(unittest.TestCase):
     def test_repr_on(self):
         switch = self._makeOne()
         self.assertEqual(repr(switch), '<ShowSwitch on>')
-        
+
     def test_repr_off(self):
         switch = self._makeOne()
         switch.stack.append(False)
@@ -77,7 +77,7 @@ class TestDeprecationProxy(WarningsSetupBase, unittest.TestCase):
     def _getTargetClass(self):
         from zope.deprecation.deprecation import DeprecationProxy
         return DeprecationProxy
-        
+
     def _makeOne(self, module):
         cls = self._getTargetClass()
         return cls(module)
@@ -91,6 +91,15 @@ class TestDeprecationProxy(WarningsSetupBase, unittest.TestCase):
             self.warnings.w,
             [('ClassFixture: hello', DeprecationWarning, 2)])
 
+    def test_deprecate_and__getattribute__string_with_custom_cls(self):
+        tests = _getTestsModule()
+        proxy = self._makeOne(tests)
+        proxy.deprecate('ClassFixture', 'hello', DummyWarning)
+        self.assertEqual(proxy.ClassFixture, ClassFixture)
+        self.assertEqual(
+            self.warnings.w,
+            [('ClassFixture: hello', DummyWarning, 2)])
+
     def test_deprecate_and__getattribute__sequence(self):
         tests = _getTestsModule()
         proxy = self._makeOne(tests)
@@ -159,10 +168,10 @@ class TestDeprecatedModule(WarningsSetupBase, unittest.TestCase):
     def _getTargetClass(self):
         from zope.deprecation.deprecation import DeprecatedModule
         return DeprecatedModule
-        
-    def _makeOne(self, module, msg):
+
+    def _makeOne(self, module, msg, *args):
         cls = self._getTargetClass()
-        return cls(module, msg)
+        return cls(module, msg, *args)
 
     def test___getattribute____class__(self):
         tests = _getTestsModule()
@@ -183,6 +192,15 @@ class TestDeprecatedModule(WarningsSetupBase, unittest.TestCase):
             [('hello', DeprecationWarning, 2)]
             )
 
+    def test___getattribute___deprecated_with_custom_cls(self):
+        tests = _getTestsModule()
+        proxy = self._makeOne(tests, 'hello', DummyWarning)
+        self.assertEqual(proxy.ClassFixture, ClassFixture)
+        self.assertEqual(
+            self.warnings.w,
+            [('hello', DummyWarning, 2)]
+            )
+
     def test___getattribute__missing(self):
         tests = _getTestsModule()
         proxy = self._makeOne(tests, 'hello')
@@ -233,10 +251,10 @@ class TestDeprecatedGetProperty(WarningsSetupBase, unittest.TestCase):
     def _getTargetClass(self):
         from zope.deprecation.deprecation import DeprecatedGetProperty
         return DeprecatedGetProperty
-        
-    def _makeOne(self, prop, msg):
+
+    def _makeOne(self, prop, msg, *args):
         cls = self._getTargetClass()
-        return cls(prop, msg)
+        return cls(prop, msg, *args)
 
     def test___get__(self):
         prop = DummyProperty()
@@ -249,6 +267,17 @@ class TestDeprecatedGetProperty(WarningsSetupBase, unittest.TestCase):
             [('hello', DeprecationWarning, 2)]
             )
 
+    def test___get___with_custom_cls(self):
+        prop = DummyProperty()
+        proxy = self._makeOne(prop, 'hello', DummyWarning)
+        self.assertEqual(proxy.__get__('inst', 'cls'), None)
+        self.assertEqual(prop.inst, 'inst')
+        self.assertEqual(prop.cls, 'cls')
+        self.assertEqual(
+            self.warnings.w,
+            [('hello', DummyWarning, 2)]
+            )
+
     def test___get__noshow(self):
         prop = DummyProperty()
         self.show.on = False
@@ -262,7 +291,7 @@ class TestDeprecatedGetSetProperty(TestDeprecatedGetProperty):
     def _getTargetClass(self):
         from zope.deprecation.deprecation import DeprecatedGetSetProperty
         return DeprecatedGetSetProperty
-        
+
     def test___set__(self):
         prop = DummyProperty()
         proxy = self._makeOne(prop, 'hello')
@@ -274,6 +303,17 @@ class TestDeprecatedGetSetProperty(TestDeprecatedGetProperty):
             [('hello', DeprecationWarning, 2)]
             )
 
+    def test___set___with_custom_cls(self):
+        prop = DummyProperty()
+        proxy = self._makeOne(prop, 'hello', DummyWarning)
+        self.assertEqual(proxy.__set__('inst', 'prop'), None)
+        self.assertEqual(prop.inst, 'inst')
+        self.assertEqual(prop.prop, 'prop')
+        self.assertEqual(
+            self.warnings.w,
+            [('hello', DummyWarning, 2)]
+            )
+
     def test___set__noshow(self):
         prop = DummyProperty()
         self.show.on = False
@@ -287,7 +327,7 @@ class TestDeprecatedSetGetDeleteProperty(TestDeprecatedGetSetProperty):
     def _getTargetClass(self):
         from zope.deprecation.deprecation import DeprecatedGetSetDeleteProperty
         return DeprecatedGetSetDeleteProperty
-        
+
     def test___delete__(self):
         prop = DummyProperty()
         proxy = self._makeOne(prop, 'hello')
@@ -298,6 +338,16 @@ class TestDeprecatedSetGetDeleteProperty(TestDeprecatedGetSetProperty):
             [('hello', DeprecationWarning, 2)]
             )
 
+    def test___delete___with_custom_cls(self):
+        prop = DummyProperty()
+        proxy = self._makeOne(prop, 'hello', DummyWarning)
+        self.assertEqual(proxy.__delete__('inst'), None)
+        self.assertEqual(prop.inst, 'inst')
+        self.assertEqual(
+            self.warnings.w,
+            [('hello', DummyWarning, 2)]
+            )
+
     def test___delete__noshow(self):
         prop = DummyProperty()
         proxy = self._makeOne(prop, 'hello')
@@ -309,9 +359,9 @@ class TestDeprecatedSetGetDeleteProperty(TestDeprecatedGetSetProperty):
             )
 
 class TestDeprecatedMethod(WarningsSetupBase, unittest.TestCase):
-    def _callFUT(self, method, message):
+    def _callFUT(self, method, message, *args):
         from zope.deprecation.deprecation import DeprecatedMethod
-        return DeprecatedMethod(method, message)
+        return DeprecatedMethod(method, message, *args)
 
     def fixture(self, a, b, c=1):
         return 'fixture'
@@ -324,6 +374,14 @@ class TestDeprecatedMethod(WarningsSetupBase, unittest.TestCase):
             [('hello', DeprecationWarning, 2)]
             )
 
+    def test_it_with_custom_cls(self):
+        result = self._callFUT(self.fixture, 'hello', DummyWarning)
+        self.assertEqual(result('a', 'b', c=2), 'fixture')
+        self.assertEqual(
+            self.warnings.w,
+            [('hello', DummyWarning, 2)]
+            )
+
     def test_it_noshow(self):
         result = self._callFUT(self.fixture, 'hello')
         self.show.on = False
@@ -339,9 +397,9 @@ class Test_deprecated(WarningsSetupBase, unittest.TestCase):
         super(Test_deprecated, self).tearDown()
         sys.modules['zope.deprecation.tests'] = self.mod
 
-    def _callFUT(self, spec, message):
+    def _callFUT(self, spec, message, *args):
         from zope.deprecation.deprecation import deprecated
-        return deprecated(spec, message)
+        return deprecated(spec, message, *args)
 
     def test_string_specifier(self):
         self._callFUT('ClassFixture', 'hello')
@@ -352,6 +410,15 @@ class Test_deprecated(WarningsSetupBase, unittest.TestCase):
             self.warnings.w,
             [('ClassFixture: hello', DeprecationWarning, 2)])
 
+    def test_string_specifier_with_custom_cls(self):
+        self._callFUT('ClassFixture', 'hello', DummyWarning)
+        mod = _getTestsModule()
+        self.assertNotEqual(mod, self.mod)
+        self.assertEqual(mod.ClassFixture, ClassFixture)
+        self.assertEqual(
+            self.warnings.w,
+            [('ClassFixture: hello', DummyWarning, 2)])
+
     def test_string_specifier_sys_modules_already_mutated(self):
         from zope.deprecation.deprecation import DeprecationProxy
         mod = _getTestsModule()
@@ -370,7 +437,7 @@ class Test_deprecated(WarningsSetupBase, unittest.TestCase):
         self.assertEqual(
             self.warnings.w,
             [('hello', DeprecationWarning, 2)])
-        
+
     def test_module_specifier(self):
         mod = _getTestsModule()
         result = self._callFUT(mod, 'hello')
@@ -388,7 +455,7 @@ class Test_deprecated(WarningsSetupBase, unittest.TestCase):
         self.assertEqual(
             self.warnings.w,
             [('hello', DeprecationWarning, 2)])
-        
+
     def test_getsetproperty_specifier(self):
         prop = DummyGetSetProperty()
         result = self._callFUT(prop, 'hello')
@@ -411,13 +478,13 @@ class Test_deprecate(WarningsSetupBase, unittest.TestCase):
     def _getTargetClass(self):
         from zope.deprecation.deprecation import deprecate
         return deprecate
-        
-    def _makeOne(self, msg):
+
+    def _makeOne(self, msg, *args):
         cls = self._getTargetClass()
-        return cls(msg)
+        return cls(msg, *args)
 
     def fixture(self):
-        return 'fixture'
+        return 42
 
     def test___call__(self):
         proxy = self._makeOne('hello')
@@ -427,17 +494,33 @@ class Test_deprecate(WarningsSetupBase, unittest.TestCase):
             self.warnings.w,
             [('hello', DeprecationWarning, 2)])
 
+    def test___call__method(self):
+        proxy = self._makeOne('method')
+        result = proxy(self.fixture)
+        self.assertEqual(result(), 42)
+        self.assertEqual(
+            self.warnings.w,
+            [('method', DeprecationWarning, 2)])
+
+    def test___call__with_custom_cls(self):
+        proxy = self._makeOne('hello', DummyWarning)
+        result = proxy(functionfixture)
+        self.assertEqual(result(), None)
+        self.assertEqual(
+            self.warnings.w,
+            [('hello', DummyWarning, 2)])
+
 class Test_moved(WarningsSetupBase, unittest.TestCase):
     def setUp(self):
         super(Test_moved, self).setUp()
 
     def tearDown(self):
         super(Test_moved, self).tearDown()
-        del _getTestsModule().__dict__['abc'] 
+        del _getTestsModule().__dict__['abc']
 
-    def _callFUT(self, to_location, unsupported_in):
+    def _callFUT(self, to_location, unsupported_in, *args):
         from zope.deprecation.deprecation import moved
-        return moved(to_location, unsupported_in)
+        return moved(to_location, unsupported_in, *args)
 
     def test_unsupported_None(self):
         self._callFUT('zope.deprecation.fixture', None)
@@ -446,6 +529,13 @@ class Test_moved(WarningsSetupBase, unittest.TestCase):
              [('zope.deprecation.tests has moved to zope.deprecation.fixture.',
                DeprecationWarning, 3)])
 
+    def test_unsupported_None_with_custom_cls(self):
+        self._callFUT('zope.deprecation.fixture', None, DummyWarning)
+        self.assertEqual(
+            self.warnings.w,
+             [('zope.deprecation.tests has moved to zope.deprecation.fixture.',
+               DummyWarning, 3)])
+
     def test_unsupported_not_None(self):
         self._callFUT('zope.deprecation.fixture', '1.3')
         self.assertEqual(
@@ -461,7 +551,7 @@ class Test_import_aliases(unittest.TestCase):
             real = getattr(sys.modules['zope.deprecation.deprecation'], name)
             alias = getattr(sys.modules['zope.deprecation'], name)
             self.assertEqual(real, alias, (real, alias))
-        
+
 class DummyWarningsModule(object):
     def __init__(self):
         self.w = []
@@ -483,6 +573,9 @@ class DummyGetSetDeleteProperty(DummyGetSetProperty):
     def __delete__(self, inst):
         self.inst = inst
 
+class DummyWarning(DeprecationWarning):
+    pass
+
 DummyProperty = DummyGetSetDeleteProperty
 
 def _getTestsModule():
